@@ -43,59 +43,66 @@ use serde_json::Value;
 
 use arrow::array::*;
 use arrow::datatypes::*;
-use arrow::error::Result;
+use arrow::error::{ArrowError, Result};
+use arrow::json::JsonSerializable;
 use arrow::record_batch::RecordBatch;
 
-fn primitive_array_to_json<T: ArrowPrimitiveType>(array: &ArrayRef) -> Vec<Value> {
-    as_primitive_array::<T>(array)
+fn primitive_array_to_json<T>(array: &ArrayRef) -> Result<Vec<Value>>
+where
+    T: ArrowPrimitiveType,
+    T::Native: JsonSerializable,
+{
+    Ok(as_primitive_array::<T>(array)
         .iter()
         .map(|maybe_value| match maybe_value {
             Some(v) => v.into_json_value().unwrap_or(Value::Null),
             None => Value::Null,
         })
-        .collect()
+        .collect())
 }
 
 fn struct_array_to_jsonmap_array(
     array: &StructArray,
     row_count: usize,
-) -> Vec<JsonMap<String, Value>> {
+) -> Result<Vec<JsonMap<String, Value>>> {
     let inner_col_names = array.column_names();
 
     let mut inner_objs = iter::repeat(JsonMap::new())
         .take(row_count)
         .collect::<Vec<JsonMap<String, Value>>>();
 
-    array
-        .columns()
-        .iter()
-        .enumerate()
-        .for_each(|(j, struct_col)| {
-            set_column_for_json_rows(&mut inner_objs, row_count, struct_col, inner_col_names[j]);
-        });
-
-    inner_objs
+    for (j, struct_col) in array.columns().iter().enumerate() {
+        set_column_for_json_rows(&mut inner_objs, row_count, struct_col, inner_col_names[j])?
+    }
+    Ok(inner_objs)
 }
 
 /// Converts an arrow [`ArrayRef`] into a `Vec` of Serde JSON [`serde_json::Value`]'s
-pub fn array_to_json_array(array: &ArrayRef) -> Vec<Value> {
+pub fn array_to_json_array(array: &ArrayRef) -> Result<Vec<Value>> {
     match array.data_type() {
-        DataType::Null => iter::repeat(Value::Null).take(array.len()).collect(),
-        DataType::Boolean => as_boolean_array(array)
+        DataType::Null => Ok(iter::repeat(Value::Null).take(array.len()).collect()),
+        DataType::Boolean => Ok(as_boolean_array(array)
             .iter()
             .map(|maybe_value| match maybe_value {
                 Some(v) => v.into(),
                 None => Value::Null,
             })
-            .collect(),
+            .collect()),
 
-        DataType::Utf8 => as_string_array(array)
+        DataType::Utf8 => Ok(as_string_array(array)
             .iter()
             .map(|maybe_value| match maybe_value {
                 Some(v) => v.into(),
                 None => Value::Null,
             })
-            .collect(),
+            .collect()),
+        DataType::LargeUtf8 => Ok(as_largestring_array(array)
+            .iter()
+            .map(|maybe_value| match maybe_value {
+                Some(v) => v.into(),
+                None => Value::Null,
+            })
+            .collect()),
         DataType::Int8 => primitive_array_to_json::<Int8Type>(array),
         DataType::Int16 => primitive_array_to_json::<Int16Type>(array),
         DataType::Int32 => primitive_array_to_json::<Int32Type>(array),
@@ -109,30 +116,27 @@ pub fn array_to_json_array(array: &ArrayRef) -> Vec<Value> {
         DataType::List(_) => as_list_array(array)
             .iter()
             .map(|maybe_value| match maybe_value {
-                Some(v) => Value::Array(array_to_json_array(&v)),
-                None => Value::Null,
+                Some(v) => Ok(Value::Array(array_to_json_array(&v)?)),
+                None => Ok(Value::Null),
             })
             .collect(),
         DataType::LargeList(_) => as_large_list_array(array)
             .iter()
             .map(|maybe_value| match maybe_value {
-                Some(v) => Value::Array(array_to_json_array(&v)),
-                None => Value::Null,
+                Some(v) => Ok(Value::Array(array_to_json_array(&v)?)),
+                None => Ok(Value::Null),
             })
             .collect(),
         DataType::Struct(_) => {
-            let jsonmaps = struct_array_to_jsonmap_array(as_struct_array(array), array.len());
-            jsonmaps.into_iter().map(Value::Object).collect()
-        }
-        _ => {
-            panic!(
-                "Unsupported datatype for array conversion: {:#?}",
-                array.data_type()
-            );
+            let jsonmaps = struct_array_to_jsonmap_array(as_struct_array(array), array.len())?;
+            Ok(jsonmaps.into_iter().map(Value::Object).collect())
         }
+        t => Err(ArrowError::JsonError(format!(
+            "data type {:?} not supported",
+            t
+        ))),
     }
 }
-
 macro_rules! set_column_by_array_type {
     ($cast_fn:ident, $col_name:ident, $rows:ident, $array:ident, $row_count:ident) => {
         let arr = $cast_fn($array);
@@ -199,7 +203,10 @@ fn set_column_by_primitive_type<T: ArrowPrimitiveType>(
     row_count: usize,
     array: &ArrayRef,
     col_name: &str,
-) {
+) where
+    T: ArrowPrimitiveType,
+    T::Native: JsonSerializable,
+{
     let primitive_arr = as_primitive_array::<T>(array);
 
     rows.iter_mut()
@@ -220,37 +227,37 @@ fn set_column_for_json_rows(
     row_count: usize,
     array: &ArrayRef,
     col_name: &str,
-) {
+) -> Result<()> {
     match array.data_type() {
         DataType::Int8 => {
-            set_column_by_primitive_type::<Int8Type>(rows, row_count, array, col_name)
+            set_column_by_primitive_type::<Int8Type>(rows, row_count, array, col_name);
         }
         DataType::Int16 => {
-            set_column_by_primitive_type::<Int16Type>(rows, row_count, array, col_name)
+            set_column_by_primitive_type::<Int16Type>(rows, row_count, array, col_name);
         }
         DataType::Int32 => {
-            set_column_by_primitive_type::<Int32Type>(rows, row_count, array, col_name)
+            set_column_by_primitive_type::<Int32Type>(rows, row_count, array, col_name);
         }
         DataType::Int64 => {
-            set_column_by_primitive_type::<Int64Type>(rows, row_count, array, col_name)
+            set_column_by_primitive_type::<Int64Type>(rows, row_count, array, col_name);
         }
         DataType::UInt8 => {
-            set_column_by_primitive_type::<UInt8Type>(rows, row_count, array, col_name)
+            set_column_by_primitive_type::<UInt8Type>(rows, row_count, array, col_name);
         }
         DataType::UInt16 => {
-            set_column_by_primitive_type::<UInt16Type>(rows, row_count, array, col_name)
+            set_column_by_primitive_type::<UInt16Type>(rows, row_count, array, col_name);
         }
         DataType::UInt32 => {
-            set_column_by_primitive_type::<UInt32Type>(rows, row_count, array, col_name)
+            set_column_by_primitive_type::<UInt32Type>(rows, row_count, array, col_name);
         }
         DataType::UInt64 => {
-            set_column_by_primitive_type::<UInt64Type>(rows, row_count, array, col_name)
+            set_column_by_primitive_type::<UInt64Type>(rows, row_count, array, col_name);
         }
         DataType::Float32 => {
-            set_column_by_primitive_type::<Float32Type>(rows, row_count, array, col_name)
+            set_column_by_primitive_type::<Float32Type>(rows, row_count, array, col_name);
         }
         DataType::Float64 => {
-            set_column_by_primitive_type::<Float64Type>(rows, row_count, array, col_name)
+            set_column_by_primitive_type::<Float64Type>(rows, row_count, array, col_name);
         }
         DataType::Null => {
             // when value is null, we still set the key
@@ -417,7 +424,7 @@ fn set_column_for_json_rows(
             );
         }
         DataType::Struct(_) => {
-            let inner_objs = struct_array_to_jsonmap_array(as_struct_array(array), row_count);
+            let inner_objs = struct_array_to_jsonmap_array(as_struct_array(array), row_count)?;
             rows.iter_mut()
                 .take(row_count)
                 .zip(inner_objs.into_iter())
@@ -430,40 +437,44 @@ fn set_column_for_json_rows(
             rows.iter_mut()
                 .zip(listarr.iter())
                 .take(row_count)
-                .for_each(|(row, maybe_value)| {
+                .try_for_each(|(row, maybe_value)| -> Result<()> {
                     if let Some(v) = maybe_value {
-                        row.insert(col_name.to_string(), Value::Array(array_to_json_array(&v)));
+                        row.insert(col_name.to_string(), Value::Array(array_to_json_array(&v)?));
                     } else {
                         row.insert(col_name.to_string(), Value::Null);
                     }
-                });
+                    Ok(())
+                })?;
         }
         DataType::LargeList(_) => {
             let listarr = as_large_list_array(array);
             rows.iter_mut()
                 .zip(listarr.iter())
                 .take(row_count)
-                .for_each(|(row, maybe_value)| {
+                .try_for_each(|(row, maybe_value)| -> Result<()> {
                     if let Some(v) = maybe_value {
-                        row.insert(col_name.to_string(), Value::Array(array_to_json_array(&v)));
+                        let val = array_to_json_array(&v)?;
+                        row.insert(col_name.to_string(), Value::Array(val));
                     }
-                });
+                    Ok(())
+                })?;
         }
         DataType::Dictionary(_, value_type) => {
             let slice = array.slice(0, row_count);
             let hydrated = arrow::compute::kernels::cast::cast(&slice, value_type)
                 .expect("cannot cast dictionary to underlying values");
-            set_column_for_json_rows(rows, row_count, &hydrated, col_name)
+            set_column_for_json_rows(rows, row_count, &hydrated, col_name)?;
         }
         _ => {
             panic!("Unsupported datatype: {:#?}", array.data_type());
         }
     }
+    Ok(())
 }
 
 /// Converts an arrow [`RecordBatch`] into a `Vec` of Serde JSON
 /// [`JsonMap`]s (objects)
-pub fn record_batches_to_json_rows(batches: &[RecordBatch]) -> Vec<JsonMap<String, Value>> {
+pub fn record_batches_to_json_rows(batches: &[RecordBatch]) -> Result<Vec<JsonMap<String, Value>>> {
     let mut rows: Vec<JsonMap<String, Value>> = iter::repeat(JsonMap::new())
         .take(batches.iter().map(|b| b.num_rows()).sum())
         .collect();
@@ -471,17 +482,17 @@ pub fn record_batches_to_json_rows(batches: &[RecordBatch]) -> Vec<JsonMap<Strin
     if !rows.is_empty() {
         let schema = batches[0].schema();
         let mut base = 0;
-        batches.iter().for_each(|batch| {
+        for batch in batches {
             let row_count = batch.num_rows();
-            batch.columns().iter().enumerate().for_each(|(j, col)| {
+            for (j, col) in batch.columns().iter().enumerate() {
                 let col_name = schema.field(j).name();
-                set_column_for_json_rows(&mut rows[base..], row_count, col, col_name);
-            });
+                set_column_for_json_rows(&mut rows[base..], row_count, col, col_name)?
+            }
             base += row_count;
-        });
+        }
     }
 
-    rows
+    Ok(rows)
 }
 
 /// This trait defines how to format a sequence of JSON objects to a
@@ -608,14 +619,16 @@ where
         }
 
         self.format.start_row(&mut self.writer, is_first_row)?;
-        self.writer.write_all(&serde_json::to_vec(row)?)?;
+        self.writer.write_all(
+            &serde_json::to_vec(row).map_err(|error| ArrowError::JsonError(error.to_string()))?,
+        )?;
         self.format.end_row(&mut self.writer)?;
         Ok(())
     }
 
     /// Convert the [`RecordBatch`] into JSON rows, and write them to the output
     pub fn write_batches(&mut self, batches: &[RecordBatch]) -> Result<()> {
-        for row in record_batches_to_json_rows(batches) {
+        for row in record_batches_to_json_rows(batches)? {
             self.write_row(&Value::Object(row))?;
         }
         Ok(())
@@ -737,10 +750,10 @@ mod tests {
         let ts_millis = ts_micros / 1000;
         let ts_secs = ts_millis / 1000;
 
-        let arr_nanos = TimestampNanosecondArray::from_opt_vec(vec![Some(ts_nanos), None], None);
-        let arr_micros = TimestampMicrosecondArray::from_opt_vec(vec![Some(ts_micros), None], None);
-        let arr_millis = TimestampMillisecondArray::from_opt_vec(vec![Some(ts_millis), None], None);
-        let arr_secs = TimestampSecondArray::from_opt_vec(vec![Some(ts_secs), None], None);
+        let arr_nanos = TimestampNanosecondArray::from(vec![Some(ts_nanos), None]);
+        let arr_micros = TimestampMicrosecondArray::from(vec![Some(ts_micros), None]);
+        let arr_millis = TimestampMillisecondArray::from(vec![Some(ts_millis), None]);
+        let arr_secs = TimestampSecondArray::from(vec![Some(ts_secs), None]);
         let arr_names = StringArray::from(vec![Some("a"), Some("b")]);
 
         let schema = Schema::new(vec![
@@ -916,7 +929,7 @@ mod tests {
             Field::new(
                 "c1",
                 DataType::Struct(vec![
-                    Field::new("c11", DataType::Int32, false),
+                    Field::new("c11", DataType::Int32, true),
                     Field::new(
                         "c12",
                         DataType::Struct(vec![Field::new("c121", DataType::Utf8, false)]),
@@ -930,7 +943,7 @@ mod tests {
 
         let c1 = StructArray::from(vec![
             (
-                Field::new("c11", DataType::Int32, false),
+                Field::new("c11", DataType::Int32, true),
                 Arc::new(Int32Array::from(vec![Some(1), None, Some(5)])) as ArrayRef,
             ),
             (
@@ -1072,7 +1085,7 @@ mod tests {
             DataType::List(Box::new(Field::new(
                 "s",
                 DataType::Struct(vec![
-                    Field::new("c11", DataType::Int32, false),
+                    Field::new("c11", DataType::Int32, true),
                     Field::new(
                         "c12",
                         DataType::Struct(vec![Field::new("c121", DataType::Utf8, false)]),
@@ -1088,7 +1101,7 @@ mod tests {
 
         let struct_values = StructArray::from(vec![
             (
-                Field::new("c11", DataType::Int32, false),
+                Field::new("c11", DataType::Int32, true),
                 Arc::new(Int32Array::from(vec![Some(1), None, Some(5)])) as ArrayRef,
             ),
             (
