@@ -1,11 +1,3 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
 use crate::arrow::datatypes::DataType;
 use crate::error::{Result, VegaFusionError};
 pub use datafusion_common::ScalarValue;
@@ -110,17 +102,21 @@ impl ScalarValueHelpers for ScalarValue {
                 // To UTC integer milliseconds (alread in UTC)
                 Value::from(*v)
             }
-            ScalarValue::TimestampSecond(Some(_v), _) => {
-                unimplemented!()
+            ScalarValue::TimestampSecond(Some(v), _) => {
+                let millis = v * 1_000;
+                Value::from(millis)
             }
-            ScalarValue::TimestampMillisecond(Some(_v), _) => {
-                unimplemented!()
+            ScalarValue::TimestampMillisecond(Some(v), _) => {
+                let millis = *v;
+                Value::from(millis)
             }
-            ScalarValue::TimestampMicrosecond(Some(_v), _) => {
-                unimplemented!()
+            ScalarValue::TimestampMicrosecond(Some(v), _) => {
+                let millis = *v / 1_000;
+                Value::from(millis)
             }
-            ScalarValue::TimestampNanosecond(Some(_v), _) => {
-                unimplemented!()
+            ScalarValue::TimestampNanosecond(Some(v), _) => {
+                let millis = *v / 1_000_000;
+                Value::from(millis)
             }
             ScalarValue::IntervalYearMonth(Some(_v)) => {
                 unimplemented!()
@@ -160,7 +156,7 @@ impl ScalarValueHelpers for ScalarValue {
             ScalarValue::UInt32(Some(e)) => *e as f64,
             ScalarValue::UInt64(Some(e)) => *e as f64,
             _ => {
-                return Err(VegaFusionError::internal(&format!(
+                return Err(VegaFusionError::internal(format!(
                     "Cannot convert {} to f64",
                     self
                 )))
@@ -174,7 +170,7 @@ impl ScalarValueHelpers for ScalarValue {
                 return Ok([v0.to_f64()?, v1.to_f64()?]);
             }
         }
-        Err(VegaFusionError::internal(&format!(
+        Err(VegaFusionError::internal(format!(
             "Cannot convert {} to [f64; 2]",
             self
         )))
@@ -185,7 +181,7 @@ impl ScalarValueHelpers for ScalarValue {
             ScalarValue::Utf8(Some(value)) => value.clone(),
             ScalarValue::LargeUtf8(Some(value)) => value.clone(),
             _ => {
-                return Err(VegaFusionError::internal(&format!(
+                return Err(VegaFusionError::internal(format!(
                     "Cannot convert {} to String",
                     self
                 )))
