@@ -1,12 +1,3 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
-use crate::data::dataset::VegaFusionDataset;
 use crate::error::{Result, ResultWithContext, VegaFusionError};
 use crate::proto::gen::tasks::{Task, TzConfig};
 use crate::spec::axis::AxisSpec;
@@ -145,9 +136,9 @@ impl ChartSpec {
     pub fn to_tasks(
         &self,
         tz_config: &TzConfig,
-        datasets: &HashMap<String, VegaFusionDataset>,
+        dataset_fingerprints: &HashMap<String, String>,
     ) -> Result<Vec<Task>> {
-        let mut visitor = MakeTasksVisitor::new(tz_config, datasets);
+        let mut visitor = MakeTasksVisitor::new(tz_config, dataset_fingerprints);
         self.walk(&mut visitor)?;
         Ok(visitor.tasks)
     }
