@@ -1,11 +1,3 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
 use crate::expression::column_usage::{ColumnUsage, DatasetsColumnUsage, VlSelectionFields};
 use crate::spec::transform::{TransformColumns, TransformSpecTrait};
 use crate::spec::values::Field;
@@ -100,6 +92,7 @@ impl TransformSpecTrait for AggregateTransformSpec {
                     | Variancep
                     | Stdev
                     | Stdevp
+                    | Median
             ) {
                 // Unsupported aggregation op
                 return false;
@@ -164,4 +157,22 @@ impl TransformSpecTrait for AggregateTransformSpec {
             TransformColumns::Unknown
         }
     }
+
+    fn local_datetime_columns_produced(
+        &self,
+        input_local_datetime_columns: &[String],
+    ) -> Vec<String> {
+        // Keep input local datetime columns that are used as grouping fields
+        self.groupby
+            .iter()
+            .filter_map(|groupby_field| {
+                let groupby_field_name = groupby_field.field();
+                if input_local_datetime_columns.contains(&groupby_field_name) {
+                    Some(groupby_field_name)
+                } else {
+                    None
+                }
+            })
+            .collect::<Vec<_>>()
+    }
 }
