@@ -1,11 +1,3 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
 use crate::error::Result;
 use crate::expression::parser::parse;
 
@@ -151,4 +143,13 @@ impl TransformSpecTrait for BinTransformSpec {
             TransformColumns::Unknown
         }
     }
+
+    fn local_datetime_columns_produced(
+        &self,
+        input_local_datetime_columns: &[String],
+    ) -> Vec<String> {
+        // Keep input local datetime columns as bin passes through all input columns and will
+        // never create a local datetime column.
+        Vec::from(input_local_datetime_columns)
+    }
 }
