@@ -1,11 +1,3 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
 use crate::expression::column_usage::{ColumnUsage, DatasetsColumnUsage, VlSelectionFields};
 use crate::spec::transform::aggregate::AggregateOpSpec;
 use crate::spec::transform::{TransformColumns, TransformSpecTrait};
@@ -51,6 +43,7 @@ impl TransformSpecTrait for JoinAggregateTransformSpec {
                     | Variancep
                     | Stdev
                     | Stdevp
+                    | Median
             ) {
                 // Unsupported aggregation op
                 return false;
@@ -103,4 +96,13 @@ impl TransformSpecTrait for JoinAggregateTransformSpec {
             TransformColumns::Unknown
         }
     }
+
+    fn local_datetime_columns_produced(
+        &self,
+        input_local_datetime_columns: &[String],
+    ) -> Vec<String> {
+        // Keep input local datetime columns as joinaggregate passes through all input columns,
+        // and the new columns created by joinaggregate will never be local datetimes
+        Vec::from(input_local_datetime_columns)
+    }
 }
