@@ -1,11 +1,3 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
 use crate::spec::transform::{TransformColumns, TransformSpecTrait};
 use serde::{Deserialize, Serialize};
 use serde_json::Value;
@@ -54,4 +46,21 @@ impl TransformSpecTrait for ProjectTransformSpec {
             TransformColumns::Unknown
         }
     }
+
+    fn local_datetime_columns_produced(
+        &self,
+        input_local_datetime_columns: &[String],
+    ) -> Vec<String> {
+        // Keep input local datetime columns that are used as projection fields
+        self.fields
+            .iter()
+            .filter_map(|project_field| {
+                if input_local_datetime_columns.contains(project_field) {
+                    Some(project_field.clone())
+                } else {
+                    None
+                }
+            })
+            .collect::<Vec<_>>()
+    }
 }
