@@ -1,12 +1,3 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
-use crate::data::dataset::VegaFusionDataset;
 use crate::data::scalar::{ScalarValue, ScalarValueHelpers};
 use crate::data::table::VegaFusionTable;
 use crate::error::{Result, VegaFusionError};
@@ -29,7 +20,6 @@ use crate::spec::values::{SignalExpressionSpec, StringOrSignalSpec};
 use crate::task_graph::graph::ScopedVariable;
 use crate::task_graph::scope::TaskScope;
 use crate::task_graph::task_value::TaskValue;
-use serde_json::Value;
 use std::collections::{HashMap, HashSet};
 use std::convert::TryFrom;
 use std::ops::Deref;
@@ -108,15 +98,15 @@ impl ChartVisitor for MakeTaskScopeVisitor {
 pub struct MakeTasksVisitor<'a> {
     pub tasks: Vec<Task>,
     pub tz_config: TzConfig,
-    pub datasets: &'a HashMap<String, VegaFusionDataset>,
+    pub dataset_fingerprints: &'a HashMap<String, String>,
 }
 
 impl<'a> MakeTasksVisitor<'a> {
-    pub fn new(tz_config: &TzConfig, datasets: &'a HashMap<String, VegaFusionDataset>) -> Self {
+    pub fn new(tz_config: &TzConfig, dataset_fingerprints: &'a HashMap<String, String>) -> Self {
         Self {
             tasks: Default::default(),
             tz_config: tz_config.clone(),
-            datasets,
+            dataset_fingerprints,
         }
     }
 }
@@ -178,8 +168,8 @@ impl<'a> ChartVisitor for MakeTasksVisitor<'a> {
             if let Url::String(url) = &proto_url {
                 if let Some(inline_name) = url.strip_prefix("vegafusion+dataset://") {
                     let inline_name = inline_name.trim().to_string();
-                    if let Some(dataset) = self.datasets.get(&inline_name) {
-                        proto_url = Url::String(format!("{}#{}", url, dataset.fingerprint()));
+                    if let Some(fingerprint) = self.dataset_fingerprints.get(&inline_name) {
+                        proto_url = Url::String(format!("{}#{}", url, fingerprint));
                     }
                 }
             }
@@ -210,7 +200,7 @@ impl<'a> ChartVisitor for MakeTasksVisitor<'a> {
                 Some(values) => VegaFusionTable::from_json(values, 1024)?,
                 None => {
                     // Treat as empty values array
-                    VegaFusionTable::from_json(&Value::Array(Vec::new()), 1)?
+                    VegaFusionTable::empty_with_ordering()
                 }
             };
 
@@ -313,13 +303,16 @@ impl<'a> UpdateVarsChartVisitor<'a> {
 /// Gather variables that can be updated by the spec (whether or not they are defined in the spec)
 impl<'a> ChartVisitor for UpdateVarsChartVisitor<'a> {
     fn visit_data(&mut self, data: &DataSpec, scope: &[u32]) -> Result<()> {
-        // Dataset is an update dependency if it's not an empty stub (without inline values)
-        if data.source.is_some()
+        if data.is_selection_store() && data.values.is_some() {
+            // Selection store with inline values not considered an update so that
+            // we will still receive updates from client to server
+        } else if data.source.is_some()
             || data.on.is_some()
             || data.url.is_some()
             || data.values.is_some()
             || !data.transform.is_empty()
         {
+            // Dataset is an update dependency if it's not an empty stub (without inline values)
             self.update_vars
                 .insert((Variable::new_data(&data.name), Vec::from(scope)));
         }
@@ -414,6 +407,14 @@ impl<'a> InputVarsChartVisitor<'a> {
 
             if let Some(MarkFacetSpec { data, .. }) = &from.facet {
                 let data_var = Variable::new_data(data);
+                // A facet data reference does not have access to datasets defined in the same scope,
+                // so drop the final scope entry to simulate the reference being one level above
+                let scope = if !scope.is_empty() {
+                    &scope[..scope.len() - 1]
+                } else {
+                    scope
+                };
+
                 let resolved = self.task_scope.resolve_scope(&data_var, scope)?;
                 self.input_vars.insert((data_var, resolved.scope));
             }
