@@ -1,11 +1,6 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
+use crate::expression::compiler::call::make_session_context;
+use crate::sql::connection::datafusion_conn::DataFusionConnection;
+use crate::sql::dataframe::SqlDataFrame;
 use crate::transform::utils::DataFrameUtils;
 use async_trait::async_trait;
 use datafusion::dataframe::DataFrame;
@@ -19,22 +14,23 @@ use vegafusion_core::error::{Result, ResultWithContext};
 
 #[async_trait]
 pub trait VegaFusionTableUtils {
-    fn from_dataframe_blocking(value: Arc<DataFrame>) -> Result<VegaFusionTable>;
-    async fn from_dataframe(value: Arc<DataFrame>) -> Result<VegaFusionTable>;
+    fn from_dataframe_blocking(value: DataFrame) -> Result<VegaFusionTable>;
+    async fn from_dataframe(value: DataFrame) -> Result<VegaFusionTable>;
     fn pretty_format(&self, max_rows: Option<usize>) -> Result<String>;
     fn to_memtable(&self) -> MemTable;
-    fn to_dataframe(&self) -> Result<Arc<DataFrame>>;
+    async fn to_dataframe(&self) -> Result<DataFrame>;
+    async fn to_sql_dataframe(&self) -> Result<Arc<SqlDataFrame>>;
 }
 
 #[async_trait]
 impl VegaFusionTableUtils for VegaFusionTable {
-    fn from_dataframe_blocking(value: Arc<DataFrame>) -> Result<Self> {
+    fn from_dataframe_blocking(value: DataFrame) -> Result<Self> {
         let schema: SchemaRef = Arc::new(value.schema().into()) as SchemaRef;
         let batches = value.block_eval()?;
         Ok(Self { schema, batches })
     }
 
-    async fn from_dataframe(value: Arc<DataFrame>) -> Result<VegaFusionTable> {
+    async fn from_dataframe(value: DataFrame) -> Result<VegaFusionTable> {
         let schema: SchemaRef = Arc::new(value.schema().into()) as SchemaRef;
         let batches = value.collect().await?;
         Ok(Self { schema, batches })
@@ -72,11 +68,21 @@ impl VegaFusionTableUtils for VegaFusionTable {
         })
     }
 
-    fn to_dataframe(&self) -> Result<Arc<DataFrame>> {
+    async fn to_dataframe(&self) -> Result<DataFrame> {
         let ctx = SessionContext::new();
         let provider = self.to_memtable();
         ctx.register_table("df", Arc::new(provider)).unwrap();
         ctx.table("df")
+            .await
             .with_context(|| "Failed to create DataFrame".to_string())
     }
+
+    async fn to_sql_dataframe(&self) -> Result<Arc<SqlDataFrame>> {
+        let ctx = make_session_context();
+        ctx.register_table("tbl", Arc::new(self.to_memtable()))?;
+        let sql_conn = DataFusionConnection::new(Arc::new(ctx));
+        Ok(Arc::new(
+            SqlDataFrame::try_new(Arc::new(sql_conn), "tbl").await?,
+        ))
+    }
 }
