@@ -1,23 +1,10 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
 use crate::data::table::VegaFusionTableUtils;
-use crate::expression::compiler::builtin_functions::date_time::date_parsing::{
-    get_datetime_udf, DateParseMode,
-};
-use crate::expression::compiler::builtin_functions::date_time::datetime::make_datetime_components_udf;
+use crate::expression::compiler::builtin_functions::date_time::datetime::MAKE_TIMESTAMPTZ;
 use crate::expression::compiler::compile;
 use crate::expression::compiler::config::CompilationConfig;
-use crate::expression::compiler::utils::{
-    cast_to, is_integer_datatype, is_string_datatype, ExprHelpers,
-};
+use crate::expression::compiler::utils::{is_integer_datatype, is_string_datatype, ExprHelpers};
 use crate::task_graph::task::TaskCall;
-use crate::transform::TransformTrait;
+
 use async_trait::async_trait;
 use datafusion::arrow::datatypes::{DataType, Field, Schema, SchemaRef};
 use datafusion::arrow::ipc::reader::{FileReader, StreamReader};
@@ -25,19 +12,25 @@ use datafusion::arrow::record_batch::RecordBatch;
 use datafusion::dataframe::DataFrame;
 use datafusion::datasource::listing::ListingTableUrl;
 use datafusion::execution::options::CsvReadOptions;
-use datafusion::logical_plan::Expr;
-use datafusion::prelude::{col, SessionContext};
+use datafusion::logical_expr::Expr;
+use datafusion::prelude::SessionContext;
+use datafusion_expr::lit;
 use std::collections::{HashMap, HashSet};
 use std::fs::File;
 use std::io::Write;
 use std::sync::Arc;
 use tokio::io::AsyncReadExt;
-use vegafusion_core::arrow::datatypes::TimeUnit;
-use vegafusion_core::data::dataset::VegaFusionDataset;
 
-use crate::data::dataset::VegaFusionDatasetUtils;
-use crate::expression::compiler::builtin_functions::date_time::local_to_utc::make_to_utc_millis_fn;
+use crate::data::dataset::VegaFusionDataset;
+use crate::expression::compiler::builtin_functions::date_time::date_to_timestamptz::DATE_TO_TIMESTAMPTZ_UDF;
+use crate::expression::compiler::builtin_functions::date_time::str_to_timestamptz::STR_TO_TIMESTAMPTZ_UDF;
+use crate::expression::compiler::builtin_functions::date_time::timestamp_to_timestamptz::TIMESTAMP_TO_TIMESTAMPTZ_UDF;
+use crate::expression::compiler::call::make_session_context;
+use crate::expression::escape::flat_col;
+use crate::sql::connection::datafusion_conn::DataFusionConnection;
+use crate::sql::dataframe::SqlDataFrame;
 use crate::task_graph::timezone::RuntimeTzConfig;
+use crate::transform::pipeline::{remove_order_col, TransformPipelineUtils};
 use vegafusion_core::data::scalar::{ScalarValue, ScalarValueHelpers};
 use vegafusion_core::data::table::VegaFusionTable;
 use vegafusion_core::error::{Result, ResultWithContext, ToExternalError, VegaFusionError};
@@ -45,9 +38,13 @@ use vegafusion_core::proto::gen::tasks::data_url_task::Url;
 use vegafusion_core::proto::gen::tasks::scan_url_format;
 use vegafusion_core::proto::gen::tasks::scan_url_format::Parse;
 use vegafusion_core::proto::gen::tasks::{DataSourceTask, DataUrlTask, DataValuesTask};
+use vegafusion_core::proto::gen::transforms::TransformPipeline;
 use vegafusion_core::task_graph::task::{InputVariable, TaskDependencies};
 use vegafusion_core::task_graph::task_value::TaskValue;
 
+use reqwest_middleware::{ClientBuilder, ClientWithMiddleware};
+use reqwest_retry::{policies::ExponentialBackoff, RetryTransientMiddleware};
+
 pub fn build_compilation_config(
     input_vars: &[InputVariable],
     values: &[TaskValue],
@@ -109,11 +106,20 @@ impl TaskCall for DataUrlTask {
         // Load data from URL
         let parse = self.format_type.as_ref().and_then(|fmt| fmt.parse.clone());
 
-        let date_mode = DateParseMode::JavaScript;
         let df = if let Some(inline_name) = url.strip_prefix("vegafusion+dataset://") {
             let inline_name = inline_name.trim().to_string();
             if let Some(inline_dataset) = inline_datasets.get(&inline_name) {
-                inline_dataset.to_dataframe()?
+                let sql_df = match inline_dataset {
+                    VegaFusionDataset::Table { table, .. } => {
+                        table.clone().with_ordering()?.to_sql_dataframe().await?
+                    }
+                    VegaFusionDataset::SqlDataFrame(sql_df) => {
+                        // TODO: if no ordering column present, create with a window expression
+                        sql_df.clone()
+                    }
+                };
+                let sql_df = process_datetimes(&parse, sql_df, &config.tz_config).await?;
+                return eval_sql_df(sql_df.clone(), &self.pipeline, &config).await;
             } else {
                 return Err(VegaFusionError::internal(format!(
                     "No inline dataset named {}",
@@ -127,34 +133,50 @@ impl TaskCall for DataUrlTask {
         } else if url.ends_with(".arrow") || url.ends_with(".feather") {
             read_arrow(&url).await?
         } else {
-            return Err(VegaFusionError::internal(&format!(
+            return Err(VegaFusionError::internal(format!(
                 "Invalid url file extension {}",
                 url
             )));
         };
 
-        let df = process_datetimes(&parse, date_mode, df, tz_config)?;
+        // Construct SqlDataFrame
+        let ctx = make_session_context();
+        ctx.register_table("tbl", Arc::new(df))?;
+        let sql_conn = DataFusionConnection::new(Arc::new(ctx));
 
-        // Apply transforms (if any)
-        let (transformed_df, output_values) = if self
-            .pipeline
-            .as_ref()
-            .map(|p| !p.transforms.is_empty())
-            .unwrap_or(false)
-        {
-            let pipeline = self.pipeline.as_ref().unwrap();
-            pipeline.eval(df, &config).await?
-        } else {
-            // No transforms
-            (df, Vec::new())
-        };
+        let sql_df = Arc::new(SqlDataFrame::try_new(Arc::new(sql_conn), "tbl").await?);
 
-        let table_value = TaskValue::Table(VegaFusionTable::from_dataframe(transformed_df).await?);
+        // Process datetime columns
+        let sql_df = process_datetimes(&parse, sql_df, &config.tz_config).await?;
 
-        Ok((table_value, output_values))
+        eval_sql_df(sql_df, &self.pipeline, &config).await
     }
 }
 
+async fn eval_sql_df(
+    sql_df: Arc<SqlDataFrame>,
+    pipeline: &Option<TransformPipeline>,
+    config: &CompilationConfig,
+) -> Result<(TaskValue, Vec<TaskValue>)> {
+    // Apply transforms (if any)
+    let (transformed_df, output_values) = if pipeline
+        .as_ref()
+        .map(|p| !p.transforms.is_empty())
+        .unwrap_or(false)
+    {
+        let pipeline = pipeline.as_ref().unwrap();
+        pipeline.eval_sql(sql_df, config).await?
+    } else {
+        // No transforms, just remove any ordering column
+        let sql_df = remove_order_col(sql_df).await?;
+        (sql_df.collect().await?, Vec::new())
+    };
+
+    let table_value = TaskValue::Table(transformed_df);
+
+    Ok((table_value, output_values))
+}
+
 lazy_static! {
     static ref BUILT_IN_DATASETS: HashSet<&'static str> = vec![
         "7zip.png",
@@ -250,76 +272,54 @@ fn check_builtin_dataset(url: String) -> String {
     }
 }
 
-fn process_datetimes(
+async fn process_datetimes(
     parse: &Option<Parse>,
-    date_mode: DateParseMode,
-    df: Arc<DataFrame>,
+    sql_df: Arc<SqlDataFrame>,
     tz_config: &Option<RuntimeTzConfig>,
-) -> Result<Arc<DataFrame>> {
+) -> Result<Arc<SqlDataFrame>> {
     // Perform specialized date parsing
     let mut date_fields: Vec<String> = Vec::new();
-    let mut df = df;
+    let mut df = sql_df;
     if let Some(scan_url_format::Parse::Object(formats)) = parse {
         for spec in &formats.specs {
             let datatype = &spec.datatype;
             if datatype.starts_with("date") || datatype.starts_with("utc") {
-                let date_mode = if datatype.starts_with("utc") {
-                    DateParseMode::Utc
-                } else {
-                    date_mode
-                };
-
-                let schema = df.schema();
+                let schema = df.schema_df();
                 if let Ok(date_field) = schema.field_with_unqualified_name(&spec.name) {
-                    date_fields.push(date_field.name().clone());
                     let dtype = date_field.data_type();
                     let date_expr = if is_string_datatype(dtype) {
-                        let datetime_udf = get_datetime_udf(date_mode, tz_config);
+                        let default_input_tz_str = tz_config
+                            .map(|tz_config| tz_config.default_input_tz.to_string())
+                            .unwrap_or_else(|| "UTC".to_string());
 
                         Expr::ScalarUDF {
-                            fun: Arc::new(datetime_udf),
-                            args: vec![col(&spec.name)],
+                            fun: Arc::new((*STR_TO_TIMESTAMPTZ_UDF).clone()),
+                            args: vec![flat_col(&spec.name), lit(default_input_tz_str)],
                         }
                     } else if is_integer_datatype(dtype) {
                         // Assume Year was parsed numerically, use local time
                         let tz_config =
                             tz_config.with_context(|| "No local timezone info provided")?;
                         Expr::ScalarUDF {
-                            fun: Arc::new(make_datetime_components_udf(tz_config.default_input_tz)),
-                            args: vec![col(&spec.name)],
-                        }
-                    } else if let DataType::Timestamp(_, tz) = dtype {
-                        match tz {
-                            Some(tz) if tz.to_lowercase() == "utc" => {
-                                let timestamp_millis = cast_to(
-                                    col(&spec.name),
-                                    &DataType::Timestamp(
-                                        TimeUnit::Millisecond,
-                                        Some("UTC".to_string()),
-                                    ),
-                                    schema,
-                                )?;
-                                cast_to(timestamp_millis, &DataType::Int64, schema).unwrap()
-                            }
-                            _ => {
-                                let timestamp_millis = cast_to(
-                                    col(&spec.name),
-                                    &DataType::Timestamp(TimeUnit::Millisecond, None),
-                                    schema,
-                                )?;
-                                // Treat as local
-                                let tz_config =
-                                    tz_config.with_context(|| "No local timezone info provided")?;
-                                Expr::ScalarUDF {
-                                    fun: Arc::new(make_to_utc_millis_fn(&tz_config)),
-                                    args: vec![timestamp_millis],
-                                }
-                            }
+                            fun: Arc::new((*MAKE_TIMESTAMPTZ).clone()),
+                            args: vec![
+                                flat_col(&spec.name),                        // year
+                                lit(0),                                      // month
+                                lit(1),                                      // day
+                                lit(0),                                      // hour
+                                lit(0),                                      // minute
+                                lit(0),                                      // second
+                                lit(0),                                      // millisecond
+                                lit(tz_config.default_input_tz.to_string()), // time zone
+                            ],
                         }
                     } else {
                         continue;
                     };
 
+                    // Add to date_fields if special date processing was performed
+                    date_fields.push(date_field.name().clone());
+
                     let mut columns: Vec<_> = schema
                         .fields()
                         .iter()
@@ -328,12 +328,12 @@ fn process_datetimes(
                             if name == &spec.name {
                                 None
                             } else {
-                                Some(col(name))
+                                Some(flat_col(name))
                             }
                         })
                         .collect();
                     columns.push(date_expr.alias(&spec.name));
-                    df = df.select(columns)?
+                    df = df.select(columns).await?
                 }
             }
         }
@@ -346,46 +346,66 @@ fn process_datetimes(
         .fields()
         .iter()
         .map(|field| {
-            if !date_fields.contains(field.name())
-                && matches!(field.data_type(), DataType::Timestamp(_, _))
-            {
+            if !date_fields.contains(field.name()) {
                 let expr = match field.data_type() {
                     DataType::Timestamp(_, tz) => match tz {
-                        Some(tz) if tz.to_lowercase() == "utc" => {
-                            let timestamp_millis = cast_to(
-                                col(field.name()),
-                                &DataType::Timestamp(
-                                    TimeUnit::Millisecond,
-                                    Some("UTC".to_string()),
-                                ),
-                                schema,
-                            )?;
-                            cast_to(timestamp_millis, &DataType::Int64, schema).unwrap()
+                        Some(tz) => {
+                            // Timestamp has explicit timezone
+                            Expr::ScalarUDF {
+                                fun: Arc::new((*TIMESTAMP_TO_TIMESTAMPTZ_UDF).clone()),
+                                args: vec![flat_col(field.name()), lit(tz.as_str())],
+                            }
                         }
                         _ => {
-                            let timestamp_millis = cast_to(
-                                col(field.name()),
-                                &DataType::Timestamp(TimeUnit::Millisecond, None),
-                                schema,
-                            )?;
+                            // Naive timestamp, interpret as default_input_tz
                             let tz_config =
                                 tz_config.with_context(|| "No local timezone info provided")?;
+
                             Expr::ScalarUDF {
-                                fun: Arc::new(make_to_utc_millis_fn(&tz_config)),
-                                args: vec![timestamp_millis],
+                                fun: Arc::new((*TIMESTAMP_TO_TIMESTAMPTZ_UDF).clone()),
+                                args: vec![
+                                    flat_col(field.name()),
+                                    lit(tz_config.default_input_tz.to_string()),
+                                ],
                             }
                         }
                     },
-                    _ => unreachable!(),
+                    DataType::Date64 => {
+                        let tz_config =
+                            tz_config.with_context(|| "No local timezone info provided")?;
+
+                        Expr::ScalarUDF {
+                            fun: Arc::new((*TIMESTAMP_TO_TIMESTAMPTZ_UDF).clone()),
+                            args: vec![
+                                flat_col(field.name()),
+                                lit(tz_config.default_input_tz.to_string()),
+                            ],
+                        }
+                    }
+                    DataType::Date32 => {
+                        let tz_config =
+                            tz_config.with_context(|| "No local timezone info provided")?;
+
+                        Expr::ScalarUDF {
+                            fun: Arc::new((*DATE_TO_TIMESTAMPTZ_UDF).clone()),
+                            args: vec![flat_col(field.name()), lit(tz_config.local_tz.to_string())],
+                        }
+                    }
+                    _ => flat_col(field.name()),
                 };
-                Ok(expr.alias(field.name()))
+
+                Ok(if matches!(expr, Expr::Alias(_, _)) {
+                    expr
+                } else {
+                    expr.alias(field.name())
+                })
             } else {
-                Ok(col(field.name()))
+                Ok(flat_col(field.name()))
             }
         })
         .collect::<Result<Vec<_>>>()?;
 
-    Ok(df.select(selection)?)
+    df.select(selection).await
 }
 
 #[async_trait]
@@ -398,10 +418,15 @@ impl TaskCall for DataValuesTask {
     ) -> Result<(TaskValue, Vec<TaskValue>)> {
         // Deserialize data into table
         let values_table = VegaFusionTable::from_ipc_bytes(&self.values)?;
+        if values_table.schema.fields.is_empty() {
+            return Ok((TaskValue::Table(values_table), Default::default()));
+        }
+
+        // Add ordering column
+        let values_table = values_table.with_ordering()?;
 
         // Get parse format for date processing
         let parse = self.format_type.as_ref().and_then(|fmt| fmt.parse.clone());
-        let date_mode = DateParseMode::JavaScript;
 
         // Apply transforms (if any)
         let (transformed_table, output_values) = if self
@@ -411,21 +436,21 @@ impl TaskCall for DataValuesTask {
             .unwrap_or(false)
         {
             let pipeline = self.pipeline.as_ref().unwrap();
-            let values_df = values_table.to_dataframe()?;
-            let values_df = process_datetimes(&parse, date_mode, values_df, tz_config)?;
 
             let config = build_compilation_config(&self.input_vars(), values, tz_config);
-            let (df, output_values) = pipeline.eval(values_df, &config).await?;
 
-            (VegaFusionTable::from_dataframe(df).await?, output_values)
+            // Process datetime columns
+            let sql_df = values_table.to_sql_dataframe().await?;
+            let sql_df = process_datetimes(&parse, sql_df, &config.tz_config).await?;
+
+            let (table, output_values) = pipeline.eval_sql(sql_df, &config).await?;
+
+            (table, output_values)
         } else {
             // No transforms
-            let values_df = values_table.to_dataframe()?;
-            let values_df = process_datetimes(&parse, date_mode, values_df, tz_config)?;
-            (
-                VegaFusionTable::from_dataframe(values_df).await?,
-                Vec::new(),
-            )
+            let values_df = values_table.to_sql_dataframe().await?;
+            let values_df = process_datetimes(&parse, values_df, tz_config).await?;
+            (values_df.collect().await?, Vec::new())
         };
 
         let table_value = TaskValue::Table(transformed_table);
@@ -453,6 +478,9 @@ impl TaskCall for DataSourceTask {
             )
         });
 
+        // Add ordering column
+        let source_table = source_table.with_ordering()?;
+
         // Apply transforms (if any)
         let (transformed_table, output_values) = if self
             .pipeline
@@ -461,9 +489,10 @@ impl TaskCall for DataSourceTask {
             .unwrap_or(false)
         {
             let pipeline = self.pipeline.as_ref().unwrap();
-            let values_df = source_table.to_dataframe()?;
-            let (df, output_values) = pipeline.eval(values_df, &config).await?;
-            (VegaFusionTable::from_dataframe(df).await?, output_values)
+            let sql_df = source_table.to_sql_dataframe().await?;
+            let (table, output_values) = pipeline.eval_sql(sql_df, &config).await?;
+
+            (table, output_values)
         } else {
             // No transforms
             (source_table, Vec::new())
@@ -474,7 +503,7 @@ impl TaskCall for DataSourceTask {
     }
 }
 
-async fn read_csv(url: String, parse: &Option<Parse>) -> Result<Arc<DataFrame>> {
+async fn read_csv(url: String, parse: &Option<Parse>) -> Result<DataFrame> {
     // Build base CSV options
     let csv_opts = if url.ends_with(".tsv") {
         CsvReadOptions::new()
@@ -488,7 +517,9 @@ async fn read_csv(url: String, parse: &Option<Parse>) -> Result<Arc<DataFrame>>
 
     if url.starts_with("http://") || url.starts_with("https://") {
         // Perform get request to collect file contents as text
-        let body = reqwest::get(url.clone())
+        let body = make_request_client()
+            .get(url.clone())
+            .send()
             .await
             .external(&format!("Failed to get URL data from {}", url))?
             .text()
@@ -512,12 +543,18 @@ async fn read_csv(url: String, parse: &Option<Parse>) -> Result<Arc<DataFrame>>
         // Load through VegaFusionTable so that temp file can be deleted
         let df = ctx.read_csv(path, csv_opts).await.unwrap();
         let table = VegaFusionTable::from_dataframe(df).await.unwrap();
-        let df = table.to_dataframe().unwrap();
+        let table = table.with_ordering()?;
+        let df = table.to_dataframe().await.unwrap();
         Ok(df)
     } else {
         let schema = build_csv_schema(&csv_opts, &url, parse).await?;
         let csv_opts = csv_opts.schema(&schema);
-        Ok(ctx.read_csv(url, csv_opts).await?)
+
+        let df = ctx.read_csv(url, csv_opts).await.unwrap();
+        let table = VegaFusionTable::from_dataframe(df).await.unwrap();
+        let table = table.with_ordering()?;
+        let df = table.to_dataframe().await.unwrap();
+        Ok(df)
     }
 }
 
@@ -528,7 +565,7 @@ async fn build_csv_schema(
 ) -> Result<SchemaRef> {
     let ctx = SessionContext::new();
     let table_path = ListingTableUrl::parse(uri.into().as_str())?;
-    let target_partitions = ctx.copied_config().target_partitions;
+    let target_partitions = ctx.copied_config().target_partitions();
     let listing_options = csv_opts.to_listing_options(target_partitions);
     let inferred_schema = listing_options
         .infer_schema(&ctx.state(), &table_path)
@@ -574,11 +611,13 @@ async fn build_csv_schema(
     Ok(SchemaRef::new(Schema::new(new_fields)))
 }
 
-async fn read_json(url: &str, batch_size: usize) -> Result<Arc<DataFrame>> {
+async fn read_json(url: &str, batch_size: usize) -> Result<DataFrame> {
     // Read to json Value from local file or url.
     let value: serde_json::Value = if url.starts_with("http://") || url.starts_with("https://") {
         // Perform get request to collect file contents as text
-        let body = reqwest::get(url)
+        let body = make_request_client()
+            .get(url)
+            .send()
             .await
             .external(&format!("Failed to get URL data from {}", url))?
             .text()
@@ -590,7 +629,7 @@ async fn read_json(url: &str, batch_size: usize) -> Result<Arc<DataFrame>> {
         // Assume local file
         let mut file = tokio::fs::File::open(url)
             .await
-            .external(&format!("Failed to open as local file: {}", url))?;
+            .external(format!("Failed to open as local file: {}", url))?;
 
         let mut json_str = String::new();
         file.read_to_string(&mut json_str)
@@ -600,14 +639,19 @@ async fn read_json(url: &str, batch_size: usize) -> Result<Arc<DataFrame>> {
         serde_json::from_str(&json_str)?
     };
 
-    VegaFusionTable::from_json(&value, batch_size)?.to_dataframe()
+    VegaFusionTable::from_json(&value, batch_size)?
+        .with_ordering()?
+        .to_dataframe()
+        .await
 }
 
-async fn read_arrow(url: &str) -> Result<Arc<DataFrame>> {
+async fn read_arrow(url: &str) -> Result<DataFrame> {
     // Read to json Value from local file or url.
     let buffer = if url.starts_with("http://") || url.starts_with("https://") {
         // Perform get request to collect file contents as text
-        reqwest::get(url)
+        make_request_client()
+            .get(url)
+            .send()
             .await
             .external(&format!("Failed to get URL data from {}", url))?
             .bytes()
@@ -617,7 +661,7 @@ async fn read_arrow(url: &str) -> Result<Arc<DataFrame>> {
         // Assume local file
         let mut file = tokio::fs::File::open(url)
             .await
-            .external(&format!("Failed to open as local file: {}", url))?;
+            .external(format!("Failed to open as local file: {}", url))?;
 
         let mut buffer: Vec<u8> = Vec::new();
         file.read_to_end(&mut buffer)
@@ -652,5 +696,16 @@ async fn read_arrow(url: &str) -> Result<Arc<DataFrame>> {
         )));
     };
 
-    VegaFusionTable::try_new(schema, batches)?.to_dataframe()
+    VegaFusionTable::try_new(schema, batches)?
+        .with_ordering()?
+        .to_dataframe()
+        .await
+}
+
+pub fn make_request_client() -> ClientWithMiddleware {
+    // Retry up to 3 times with increasing intervals between attempts.
+    let retry_policy = ExponentialBackoff::builder().build_with_max_retries(3);
+    ClientBuilder::new(reqwest::Client::new())
+        .with(RetryTransientMiddleware::new_with_policy(retry_policy))
+        .build()
 }
