@@ -1,13 +1,6 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
 use datafusion::arrow::datatypes::DataType;
-use datafusion::logical_plan::{lit, DFSchema, Expr};
+use datafusion::common::DFSchema;
+use datafusion::logical_expr::{lit, Expr};
 use datafusion::scalar::ScalarValue;
 use itertools::Itertools;
 use std::collections::HashMap;
@@ -23,6 +16,7 @@ use crate::expression::compiler::builtin_functions::data::vl_selection_test::{
     SelectionRow, SelectionType,
 };
 
+use crate::task_graph::timezone::RuntimeTzConfig;
 use vegafusion_core::proto::gen::{
     expression::expression::Expr as ProtoExpr, expression::Expression, expression::Literal,
 };
@@ -32,7 +26,7 @@ use super::vl_selection_test::Op;
 pub fn parse_args(args: &[Expression]) -> Result<Op> {
     let n = args.len();
     if !(0..=1).contains(&n) {
-        return Err(VegaFusionError::internal(&format!(
+        return Err(VegaFusionError::internal(format!(
             "vlSelectionResolve requires 1 or 2 arguments. Received {}",
             n
         )));
@@ -49,7 +43,7 @@ pub fn parse_args(args: &[Expression]) -> Result<Op> {
                 Op::from_str(value.as_str()).unwrap()
             }
             _ => {
-                return Err(VegaFusionError::internal(&format!(
+                return Err(VegaFusionError::internal(format!(
                     "The second argument to vlSelectionResolve, if provided, must be either 'union' or 'intersect'. \
                     Received {}", arg1
                 )))
@@ -63,6 +57,7 @@ pub fn vl_selection_resolve_fn(
     table: &VegaFusionTable,
     args: &[Expression],
     _schema: &DFSchema,
+    _tz_config: &RuntimeTzConfig,
 ) -> Result<Expr> {
     // Validate args and get operation
     let _op = parse_args(args)?;
@@ -108,7 +103,7 @@ pub fn vl_selection_resolve_fn(
                             // ])), Box::new(DataType::Float64))
                         }
                         v => {
-                            return Err(VegaFusionError::internal(&format!(
+                            return Err(VegaFusionError::internal(format!(
                                 "values must be a two-element array. Found {}",
                                 v
                             )))
