@@ -1,28 +1,29 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
 use crate::expression::compiler::utils::{
-    cast_to, is_float_datatype, is_integer_datatype, is_string_datatype, ExprHelpers,
+    cast_to, is_float_datatype, is_integer_datatype, is_numeric_datatype, is_string_datatype,
+    ExprHelpers,
 };
-use datafusion::logical_plan::{ceil, DFSchema, ExprSchemable};
-use datafusion::logical_plan::{lit, Expr};
-use datafusion::prelude::col;
+use datafusion::logical_expr::{ceil, lit, Expr, ExprSchemable};
 use std::collections::HashMap;
 use std::convert::TryFrom;
 
+use datafusion::common::DFSchema;
+use datafusion_expr::expr::Case;
+use datafusion_expr::Between;
 use std::str::FromStr;
-use vegafusion_core::arrow::datatypes::DataType;
+use std::sync::Arc;
+use vegafusion_core::arrow::datatypes::{DataType, TimeUnit};
 use vegafusion_core::data::scalar::ScalarValue;
 use vegafusion_core::error::{Result, ResultWithContext, VegaFusionError};
 use vegafusion_core::proto::gen::{
     expression::expression::Expr as ProtoExpr, expression::Expression, expression::Literal,
 };
 
+use crate::expression::compiler::builtin_functions::date_time::str_to_timestamptz::{
+    parse_datetime, STR_TO_TIMESTAMPTZ_UDF,
+};
+use crate::expression::compiler::builtin_functions::date_time::time::TIMESTAMPTZ_TO_EPOCH_MS;
+use crate::expression::escape::flat_col;
+use crate::task_graph::timezone::RuntimeTzConfig;
 use vegafusion_core::data::table::VegaFusionTable;
 use vegafusion_core::proto::gen::expression::literal::Value;
 
@@ -47,7 +48,7 @@ impl FromStr for Op {
             "union" => Self::Union,
             "intersect" => Self::Intersect,
             _ => {
-                return Err(VegaFusionError::internal(&format!(
+                return Err(VegaFusionError::internal(format!(
                     "Invalid vlSelection operation: {}",
                     s
                 )))
@@ -90,7 +91,7 @@ impl FromStr for SelectionType {
             "R-LE" => Self::RangeLe,
             "R-RE" => Self::RangeRe,
             _ => {
-                return Err(VegaFusionError::internal(&format!(
+                return Err(VegaFusionError::internal(format!(
                     "Invalid selection type: {}",
                     s
                 )))
@@ -120,20 +121,47 @@ pub struct FieldSpec {
 }
 
 impl FieldSpec {
-    pub fn to_test_expr(&self, values: &ScalarValue, schema: &DFSchema) -> Result<Expr> {
-        let field_col = col(&self.field);
+    pub fn to_test_expr(
+        &self,
+        values: &ScalarValue,
+        schema: &DFSchema,
+        default_input_tz: &str,
+    ) -> Result<Expr> {
+        let field_col = flat_col(&self.field);
+
+        // Convert timestamp column to integer milliseconds before comparisons.
+        let field_col = if matches!(
+            field_col.get_type(schema)?,
+            DataType::Timestamp(TimeUnit::Millisecond, _)
+        ) {
+            Expr::ScalarUDF {
+                fun: Arc::new((*TIMESTAMPTZ_TO_EPOCH_MS).clone()),
+                args: vec![field_col],
+            }
+        } else {
+            field_col
+        };
+
         let expr = match self.typ {
             SelectionType::Enum => {
-                let list_values: Vec<_> = if let ScalarValue::List(Some(elements), _) = &values {
-                    // values already a list
-                    elements.iter().map(|el| lit(el.clone())).collect()
+                let field_type = field_col.get_type(schema)?;
+                let list_scalars = if let ScalarValue::List(Some(elements), _) = &values {
+                    elements.clone()
                 } else {
                     // convert values to single element list
-                    vec![lit(values.clone())]
+                    vec![values.clone()]
                 };
+
+                let list_exprs = list_scalars
+                    .into_iter()
+                    .map(|scalar| {
+                        Self::cast_test_scalar(scalar, &field_type, schema, default_input_tz)
+                    })
+                    .collect::<Result<Vec<_>>>()?;
+
                 Expr::InList {
                     expr: Box::new(field_col),
-                    list: list_values,
+                    list: list_exprs,
                     negated: false,
                 }
             }
@@ -154,35 +182,45 @@ impl FieldSpec {
 
                 let (low, high) = match &values {
                     ScalarValue::List(Some(elements), _) if elements.len() == 2 => {
-                        // Don't assume elements are in ascending order
-                        let first = lit(elements[0].clone());
-                        let second = lit(elements[1].clone());
+                        let first = Self::cast_test_scalar(
+                            elements[0].clone(),
+                            &field_dtype,
+                            schema,
+                            default_input_tz,
+                        )?;
+                        let second = Self::cast_test_scalar(
+                            elements[1].clone(),
+                            &field_dtype,
+                            schema,
+                            default_input_tz,
+                        )?;
 
+                        // Don't assume elements are in ascending order
                         // Compute min and max values with Case expression
-                        let low = Expr::Case {
+                        let low = Expr::Case(Case {
                             expr: None,
                             when_then_expr: vec![(
                                 Box::new(first.clone().lt_eq(second.clone())),
                                 Box::new(first.clone()),
                             )],
                             else_expr: Some(Box::new(second.clone())),
-                        }
+                        })
                         .eval_to_scalar()?;
 
-                        let high = Expr::Case {
+                        let high = Expr::Case(Case {
                             expr: None,
                             when_then_expr: vec![(
                                 Box::new(first.clone().lt_eq(second.clone())),
                                 Box::new(second),
                             )],
                             else_expr: Some(Box::new(first)),
-                        }
+                        })
                         .eval_to_scalar()?;
 
                         (lit(low), lit(high))
                     }
                     v => {
-                        return Err(VegaFusionError::internal(&format!(
+                        return Err(VegaFusionError::internal(format!(
                             "values must be a two-element array. Found {}",
                             v
                         )))
@@ -205,12 +243,12 @@ impl FieldSpec {
                 let high = cast_to(high, &field_dtype, schema)?;
 
                 match self.typ {
-                    SelectionType::RangeInc => Expr::Between {
+                    SelectionType::RangeInc => Expr::Between(Between {
                         expr: Box::new(field_col),
                         negated: false,
                         low: Box::new(low),
                         high: Box::new(high),
-                    },
+                    }),
                     SelectionType::RangeExc => low.lt(field_col.clone()).and(field_col.lt(high)),
                     SelectionType::RangeLe => low.lt(field_col.clone()).and(field_col.lt_eq(high)),
                     SelectionType::RangeRe => low.lt_eq(field_col.clone()).and(field_col.lt(high)),
@@ -223,6 +261,37 @@ impl FieldSpec {
 
         Ok(expr)
     }
+
+    fn cast_test_scalar(
+        scalar: ScalarValue,
+        field_type: &DataType,
+        schema: &DFSchema,
+        default_input_tz: &str,
+    ) -> Result<Expr> {
+        match scalar {
+            ScalarValue::Utf8(Some(s))
+                if parse_datetime(&s, &Some(chrono_tz::UTC)).is_some()
+                    && is_numeric_datatype(field_type) =>
+            {
+                let timestamp_expr = Expr::ScalarUDF {
+                    fun: Arc::new((*STR_TO_TIMESTAMPTZ_UDF).clone()),
+                    args: vec![lit(s), lit(default_input_tz)],
+                };
+                let ms_expr = Expr::ScalarUDF {
+                    fun: Arc::new((*TIMESTAMPTZ_TO_EPOCH_MS).clone()),
+                    args: vec![timestamp_expr],
+                };
+                cast_to(ms_expr, field_type, schema)
+            }
+            _ => {
+                if is_numeric_datatype(field_type) && !is_numeric_datatype(&scalar.get_datatype()) {
+                    cast_to(lit(scalar), field_type, schema)
+                } else {
+                    Ok(lit(scalar))
+                }
+            }
+        }
+    }
 }
 
 impl TryFrom<ScalarValue> for FieldSpec {
@@ -246,7 +315,7 @@ impl TryFrom<ScalarValue> for FieldSpec {
                     Some(ScalarValue::Utf8(Some(field))) => field.clone(),
                     _ => {
                         return Err(VegaFusionError::internal(
-                            &"Expected field to be a string".to_string(),
+                            "Expected field to be a string".to_string(),
                         ))
                     }
                 };
@@ -260,7 +329,7 @@ impl TryFrom<ScalarValue> for FieldSpec {
                 Ok(Self { field, typ })
             }
             _ => Err(VegaFusionError::internal(
-                &"Expected selection field specification to be an object".to_string(),
+                "Expected selection field specification to be an object".to_string(),
             )),
         }
     }
@@ -304,10 +373,10 @@ pub struct SelectionRow {
 }
 
 impl SelectionRow {
-    pub fn to_expr(&self, schema: &DFSchema) -> Result<Expr> {
+    pub fn to_expr(&self, schema: &DFSchema, default_input_tz: &str) -> Result<Expr> {
         let mut exprs: Vec<Expr> = Vec::new();
         for (field, value) in self.fields.iter().zip(self.values.iter()) {
-            exprs.push(field.to_test_expr(value, schema)?);
+            exprs.push(field.to_test_expr(value, schema, default_input_tz)?);
         }
 
         // Take conjunction of expressions
@@ -335,7 +404,7 @@ impl TryFrom<ScalarValue> for SelectionRow {
                     Some(ScalarValue::List(Some(elements), _)) => elements.clone(),
                     _ => {
                         return Err(VegaFusionError::internal(
-                            &"Expected 'values' to be an array".to_string(),
+                            "Expected 'values' to be an array".to_string(),
                         ))
                     }
                 };
@@ -361,7 +430,7 @@ impl TryFrom<ScalarValue> for SelectionRow {
 
                 // Validate lengths
                 if values.len() != fields.len() {
-                    return Err(VegaFusionError::internal(&format!(
+                    return Err(VegaFusionError::internal(format!(
                         "Length of selection fields ({}) must match that of selection values ({})\nfield: {:?}\nvalues: {:?}",
                         fields.len(),
                         values.len(),
@@ -386,7 +455,7 @@ impl TryFrom<ScalarValue> for SelectionRow {
 fn parse_args(args: &[Expression]) -> Result<Op> {
     let n = args.len();
     if !(1..=2).contains(&n) {
-        return Err(VegaFusionError::internal(&format!(
+        return Err(VegaFusionError::internal(format!(
             "vlSelectionTest requires 2 or 3 arguments. Received {}",
             n
         )));
@@ -399,7 +468,7 @@ fn parse_args(args: &[Expression]) -> Result<Op> {
             // All good
         }
         arg => {
-            return Err(VegaFusionError::internal(&format!(
+            return Err(VegaFusionError::internal(format!(
                 "The second argument to vlSelectionTest must be datum. Received {:?}",
                 arg
             )))
@@ -417,7 +486,7 @@ fn parse_args(args: &[Expression]) -> Result<Op> {
                 Op::from_str(value.as_str()).unwrap()
             }
             _ => {
-                return Err(VegaFusionError::internal(&format!(
+                return Err(VegaFusionError::internal(format!(
                     "The third argument to vlSelectionTest, if provided, must be either 'union' or 'intersect'. \
                     Received {}", arg1
                 )))
@@ -431,6 +500,7 @@ pub fn vl_selection_test_fn(
     table: &VegaFusionTable,
     args: &[Expression],
     schema: &DFSchema,
+    tz_config: &RuntimeTzConfig,
 ) -> Result<Expr> {
     // Validate args and get operation
     let op = parse_args(args)?;
@@ -446,7 +516,7 @@ pub fn vl_selection_test_fn(
     let mut exprs: Vec<Expr> = Vec::new();
     for row in rows {
         let row_spec = SelectionRow::try_from(row)?;
-        exprs.push(row_spec.to_expr(schema)?)
+        exprs.push(row_spec.to_expr(schema, &tz_config.default_input_tz.to_string())?)
     }
 
     // Combine expressions according to op
