@@ -1,14 +1,7 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
 use crate::expression::compiler::utils::{cast_to, is_numeric_datatype};
 use datafusion::arrow::datatypes::DataType;
-use datafusion::logical_plan::{DFSchema, Expr, ExprSchemable};
+use datafusion::common::DFSchema;
+use datafusion::logical_expr::{Expr, ExprSchemable};
 use vegafusion_core::error::{Result, ResultWithContext, VegaFusionError};
 
 pub fn to_number_transform(args: &[Expr], schema: &DFSchema) -> Result<Expr> {
