@@ -1,15 +1,7 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
 use datafusion::arrow::array::{ArrayRef, BooleanArray};
 use datafusion::arrow::datatypes::{DataType, Schema};
 use datafusion::arrow::record_batch::RecordBatch;
-use datafusion::logical_plan::{and, Column, DFSchema, Expr, ExprSchemable};
+use datafusion::logical_expr::{Expr, ExprSchemable};
 use datafusion::physical_plan::planner::DefaultPhysicalPlanner;
 use datafusion::physical_plan::{ColumnarValue, PhysicalExpr, PhysicalPlanner};
 use datafusion::scalar::ScalarValue;
@@ -17,9 +9,13 @@ use datafusion::scalar::ScalarValue;
 use std::collections::HashSet;
 use std::convert::TryFrom;
 
-use datafusion::execution::context::{default_session_builder, SessionState};
+use datafusion::common::{Column, DFSchema};
+use datafusion::error::DataFusionError;
+use datafusion::execution::context::{default_session_builder, ExecutionProps, SessionState};
+use datafusion::optimizer::simplify_expressions::SimplifyInfo;
+use datafusion_expr::expr::Cast;
 use datafusion_expr::utils::expr_to_columns;
-use datafusion_expr::BuiltinScalarFunction;
+use datafusion_expr::{coalesce, lit, BuiltinScalarFunction};
 use std::sync::Arc;
 use vegafusion_core::error::{Result, ResultWithContext, VegaFusionError};
 
@@ -79,27 +75,35 @@ pub fn is_string_datatype(dtype: &DataType) -> bool {
 
 /// get datatype for expression
 pub fn data_type(value: &Expr, schema: &DFSchema) -> Result<DataType> {
-    value
-        .get_type(schema)
-        .with_context(|| format!("Failed to infer datatype of expression: {:?}", value))
+    value.get_type(schema).with_context(|| {
+        format!(
+            "Failed to infer datatype of expression: {:?}\nschema: {:?}",
+            value, schema
+        )
+    })
 }
 
 /// Cast an expression to boolean if not already boolean
 pub fn to_boolean(value: Expr, schema: &DFSchema) -> Result<Expr> {
     let dtype = data_type(&value, schema)?;
     let boolean_value = if matches!(dtype, DataType::Boolean) {
-        and(Expr::IsNotNull(Box::new(value.clone())), value)
+        coalesce(vec![value, lit(false)])
+    } else if matches!(
+        dtype,
+        DataType::Int8 | DataType::Int16 | DataType::Int32 | DataType::Int64
+    ) {
+        coalesce(vec![value.not_eq(lit(0)), lit(false)])
     } else {
         // TODO: JavaScript falsey cast
         //  - empty string to false
         //  - NaN to false
-        and(
-            Expr::Cast {
-                expr: Box::new(value.clone()),
+        coalesce(vec![
+            Expr::Cast(Cast {
+                expr: Box::new(value),
                 data_type: DataType::Boolean,
-            },
-            Expr::IsNotNull(Box::new(value)),
-        )
+            }),
+            lit(false),
+        ])
     };
 
     Ok(boolean_value)
@@ -118,10 +122,10 @@ pub fn to_numeric(value: Expr, schema: &DFSchema) -> Result<Expr> {
         }
     } else {
         // Cast non-numeric types (like UTF-8) to Float64
-        Expr::Cast {
+        Expr::Cast(Cast {
             expr: Box::new(value),
             data_type: DataType::Float64,
-        }
+        })
     };
 
     Ok(numeric_value)
@@ -133,10 +137,10 @@ pub fn to_string(value: Expr, schema: &DFSchema) -> Result<Expr> {
     let utf8_value = if dtype == DataType::Utf8 || dtype == DataType::LargeUtf8 {
         value
     } else {
-        Expr::Cast {
+        Expr::Cast(Cast {
             expr: Box::new(value),
             data_type: DataType::Utf8,
-        }
+        })
     };
 
     Ok(utf8_value)
@@ -156,10 +160,10 @@ pub fn cast_to(value: Expr, cast_dtype: &DataType, schema: &DFSchema) -> Result<
         Ok(value)
     } else {
         // Cast non-numeric types (like UTF-8) to Float64
-        Ok(Expr::Cast {
+        Ok(Expr::Cast(Cast {
             expr: Box::new(value),
             data_type: cast_dtype.clone(),
-        })
+        }))
     }
 }
 
@@ -189,7 +193,7 @@ impl ExprHelpers for Expr {
 
     fn eval_to_scalar(&self) -> Result<ScalarValue> {
         if !self.columns()?.is_empty() {
-            return Err(VegaFusionError::compilation(&format!(
+            return Err(VegaFusionError::compilation(format!(
                 "Cannot eval_to_scalar for Expr with column references: {:?}",
                 self
             )));
@@ -205,7 +209,7 @@ impl ExprHelpers for Expr {
             ColumnarValue::Scalar(scalar) => Ok(scalar),
             ColumnarValue::Array(array) => {
                 if array.len() != 1 {
-                    return Err(VegaFusionError::compilation(&format!(
+                    return Err(VegaFusionError::compilation(format!(
                         "Unexpected non-scalar array result when evaluate expr: {:?}",
                         self
                     )));
@@ -239,3 +243,40 @@ impl ExprHelpers for Expr {
         Ok(col_result)
     }
 }
+
+/// In order to simplify expressions, DataFusion must have information
+/// about the expressions.
+///
+/// You can provide that information using DataFusion [DFSchema]
+/// objects or from some other implemention
+pub struct VfSimplifyInfo {
+    /// The input schema
+    schema: DFSchema,
+
+    /// Execution specific details needed for constant evaluation such
+    /// as the current time for `now()` and [VariableProviders]
+    execution_props: ExecutionProps,
+}
+
+impl SimplifyInfo for VfSimplifyInfo {
+    fn is_boolean_type(&self, expr: &Expr) -> std::result::Result<bool, DataFusionError> {
+        Ok(matches!(expr.get_type(&self.schema)?, DataType::Boolean))
+    }
+
+    fn nullable(&self, expr: &Expr) -> std::result::Result<bool, DataFusionError> {
+        expr.nullable(&self.schema)
+    }
+
+    fn execution_props(&self) -> &ExecutionProps {
+        &self.execution_props
+    }
+}
+
+impl From<DFSchema> for VfSimplifyInfo {
+    fn from(schema: DFSchema) -> Self {
+        Self {
+            schema,
+            execution_props: ExecutionProps::new(),
+        }
+    }
+}
