@@ -1,16 +1,10 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
 use async_recursion::async_recursion;
 use std::collections::HashMap;
 use vegafusion_core::error::{Result, ResultWithContext, ToExternalError, VegaFusionError};
 use vegafusion_core::task_graph::task_value::TaskValue;
 
+use crate::data::dataset::VegaFusionDataset;
+use crate::pre_transform::destringify_selection_datetimes::destringify_selection_datetimes;
 use crate::task_graph::cache::VegaFusionCache;
 use crate::task_graph::task::TaskCall;
 use crate::task_graph::timezone::RuntimeTzConfig;
@@ -20,7 +14,6 @@ use serde_json::Value;
 use std::convert::{TryFrom, TryInto};
 use std::panic::AssertUnwindSafe;
 use std::sync::Arc;
-use vegafusion_core::data::dataset::VegaFusionDataset;
 use vegafusion_core::planning::plan::{PlannerConfig, SpecPlan};
 use vegafusion_core::planning::watch::{ExportUpdate, ExportUpdateNamespace};
 use vegafusion_core::proto::gen::errors::error::Errorkind;
@@ -183,14 +176,17 @@ impl TaskGraphRuntime {
         &self,
         request: PreTransformSpecRequest,
     ) -> Result<PreTransformSpecResult> {
-        // Get row limit
-        let row_limit = request.opts.as_ref().and_then(|opts| opts.row_limit);
-
-        // Extract and deserialize inline datasets
-        let inline_pretransform_datasets = request
-            .opts
-            .map(|opts| opts.inline_datasets)
-            .unwrap_or_default();
+        // Extract options
+        let (row_limit, preserve_interactivity, inline_pretransform_datasets) =
+            if let Some(opts) = request.opts {
+                (
+                    opts.row_limit,
+                    opts.preserve_interactivity,
+                    opts.inline_datasets,
+                )
+            } else {
+                (None, true, Default::default())
+            };
 
         let inline_datasets = inline_pretransform_datasets
             .iter()
@@ -201,40 +197,65 @@ impl TaskGraphRuntime {
             .collect::<Result<HashMap<_, _>>>()?;
 
         // Parse spec
-        let spec_string = request.spec;
+        let spec: ChartSpec = serde_json::from_str(&request.spec)?;
         let local_tz = request.local_tz;
         let output_tz = request.output_tz;
 
-        self.pre_transform_spec(
-            &spec_string,
-            &local_tz,
-            &output_tz,
-            row_limit,
-            inline_datasets,
-        )
-        .await
+        let (transformed_spec, warnings) = self
+            .pre_transform_spec(
+                &spec,
+                &local_tz,
+                &output_tz,
+                row_limit,
+                preserve_interactivity,
+                inline_datasets,
+            )
+            .await?;
+
+        // Build result
+        let response = PreTransformSpecResult {
+            result: Some(pre_transform_spec_result::Result::Response(
+                PreTransformSpecResponse {
+                    spec: serde_json::to_string(&transformed_spec)
+                        .expect("Failed to convert chart spec to string"),
+                    warnings,
+                },
+            )),
+        };
+
+        Ok(response)
     }
 
     pub async fn pre_transform_spec(
         &self,
-        spec: &str,
+        spec: &ChartSpec,
         local_tz: &str,
         default_input_tz: &Option<String>,
         row_limit: Option<u32>,
+        preserve_interactivity: bool,
         inline_datasets: HashMap<String, VegaFusionDataset>,
-    ) -> Result<PreTransformSpecResult> {
-        let spec: ChartSpec =
-            serde_json::from_str(spec).with_context(|| "Failed to parse spec".to_string())?;
+    ) -> Result<(ChartSpec, Vec<PreTransformSpecWarning>)> {
+        let input_spec = spec;
 
         // Create spec plan
         let plan = SpecPlan::try_new(
-            &spec,
+            spec,
             &PlannerConfig {
                 stringify_local_datetimes: true,
                 extract_inline_data: true,
+                allow_client_to_server_comms: !preserve_interactivity,
                 ..Default::default()
             },
         )?;
+        // println!("pre transform client_spec: {}", serde_json::to_string_pretty(&plan.client_spec).unwrap());
+        // println!("pre transform server_spec: {}", serde_json::to_string_pretty(&plan.server_spec).unwrap());
+        // println!("pre transform comm plan: {:#?}", plan.comm_plan);
+
+        // Extract inline dataset fingerprints
+        let dataset_fingerprints = inline_datasets
+            .iter()
+            .map(|(k, ds)| (k.clone(), ds.fingerprint()))
+            .collect::<HashMap<_, _>>();
 
         // Create task graph for server spec
         let tz_config = TzConfig {
@@ -244,7 +265,7 @@ impl TaskGraphRuntime {
         let task_scope = plan.server_spec.to_task_scope().unwrap();
         let tasks = plan
             .server_spec
-            .to_tasks(&tz_config, &inline_datasets)
+            .to_tasks(&tz_config, &dataset_fingerprints)
             .unwrap();
         let task_graph = TaskGraph::new(tasks, &task_scope).unwrap();
         let task_graph_mapping = task_graph.build_mapping();
@@ -285,8 +306,25 @@ impl TaskGraphRuntime {
                 }
                 ExportUpdateNamespace::Data => {
                     let data = spec.get_nested_data_mut(&scope, name)?;
-                    // Handle row_limit
-                    let value = if let Value::Array(values) = &export_update.value {
+
+                    // If the input dataset includes inline values and no transforms,
+                    // copy the input JSON directly to avoid the case where round-tripping
+                    // through Arrow homogenizes mixed type arrays.
+                    // E.g. round tripping may turn [1, "two"] into ["1", "two"]
+                    let input_values =
+                        input_spec
+                            .get_nested_data(&scope, name)
+                            .ok()
+                            .and_then(|data| {
+                                if data.transform.is_empty() {
+                                    data.values.clone()
+                                } else {
+                                    None
+                                }
+                            });
+                    let value = if let Some(input_values) = input_values {
+                        input_values
+                    } else if let Value::Array(values) = &export_update.value {
                         if let Some(row_limit) = row_limit {
                             let row_limit = row_limit as usize;
                             if values.len() > row_limit {
@@ -311,6 +349,9 @@ impl TaskGraphRuntime {
             }
         }
 
+        // Destringify datetime strings in selection store datasets
+        destringify_selection_datetimes(&mut spec)?;
+
         // Build warnings
         let mut warnings: Vec<PreTransformSpecWarning> = Vec::new();
 
@@ -354,24 +395,16 @@ impl TaskGraphRuntime {
             });
         }
 
-        // Build result
-        let response = PreTransformSpecResult {
-            result: Some(pre_transform_spec_result::Result::Response(
-                PreTransformSpecResponse {
-                    spec: serde_json::to_string(&spec)
-                        .expect("Failed to convert chart spec to string"),
-                    warnings,
-                },
-            )),
-        };
-
-        Ok(response)
+        Ok((spec, warnings))
     }
 
     pub async fn pre_transform_values_request(
         &self,
         request: PreTransformValuesRequest,
     ) -> Result<PreTransformValuesResult> {
+        // Extract row limit
+        let row_limit = request.opts.as_ref().and_then(|opts| opts.row_limit);
+
         // Extract and deserialize inline datasets
         let inline_pretransform_datasets = request
             .opts
@@ -398,15 +431,17 @@ impl TaskGraphRuntime {
 
         // Parse spec
         let spec_string = request.spec;
+        let spec: ChartSpec = serde_json::from_str(&spec_string)?;
         let local_tz = request.local_tz;
         let default_input_tz = request.default_input_tz;
 
         let (values, warnings) = self
             .pre_transform_values(
-                &spec_string,
+                &spec,
                 variables.as_slice(),
                 &local_tz,
                 &default_input_tz,
+                row_limit,
                 inline_datasets,
             )
             .await?;
@@ -439,15 +474,13 @@ impl TaskGraphRuntime {
 
     pub async fn pre_transform_values(
         &self,
-        spec: &str,
+        spec: &ChartSpec,
         variables: &[ScopedVariable],
         local_tz: &str,
         default_input_tz: &Option<String>,
+        row_limit: Option<u32>,
         inline_datasets: HashMap<String, VegaFusionDataset>,
     ) -> Result<(Vec<TaskValue>, Vec<PreTransformValuesWarning>)> {
-        let spec: ChartSpec =
-            serde_json::from_str(spec).with_context(|| "Failed to parse spec".to_string())?;
-
         // Check that requested variables exist
         for var in variables {
             let scope = var.1.as_slice();
@@ -479,16 +512,23 @@ impl TaskGraphRuntime {
 
         // Create spec plan
         let plan = SpecPlan::try_new(
-            &spec,
+            spec,
             &PlannerConfig {
-                stringify_local_datetimes: true,
+                stringify_local_datetimes: false,
                 extract_inline_data: true,
                 split_domain_data: false,
                 projection_pushdown: false,
+                allow_client_to_server_comms: true,
                 ..Default::default()
             },
         )?;
 
+        // Extract inline dataset fingerprints
+        let dataset_fingerprints = inline_datasets
+            .iter()
+            .map(|(k, ds)| (k.clone(), ds.fingerprint()))
+            .collect::<HashMap<_, _>>();
+
         // Create task graph for server spec
         let tz_config = TzConfig {
             local_tz: local_tz.to_string(),
@@ -497,7 +537,7 @@ impl TaskGraphRuntime {
         let task_scope = plan.server_spec.to_task_scope().unwrap();
         let tasks = plan
             .server_spec
-            .to_tasks(&tz_config, &inline_datasets)
+            .to_tasks(&tz_config, &dataset_fingerprints)
             .unwrap();
         let task_graph = TaskGraph::new(tasks, &task_scope).unwrap();
         let task_graph_mapping = task_graph.build_mapping();
@@ -533,6 +573,25 @@ impl TaskGraphRuntime {
                     inline_datasets.clone(),
                 )
                 .await?;
+
+            // Apply row_limit
+            let value = if let (Some(row_limit), TaskValue::Table(table)) = (row_limit, &value) {
+                if table.num_rows() > row_limit as usize {
+                    warnings.push(PreTransformValuesWarning {
+                        warning_type: Some(ValuesWarningType::RowLimit(
+                            PreTransformRowLimitWarning {
+                                datasets: vec![var.0.clone()],
+                            },
+                        )),
+                    });
+                    TaskValue::Table(table.head(row_limit as usize))
+                } else {
+                    value
+                }
+            } else {
+                value
+            };
+
             values.push(value);
         }
 
@@ -595,7 +654,7 @@ async fn get_or_compute_node_value(
                     let mut value = match value {
                         Ok(value) => value?,
                         Err(join_err) => {
-                            return Err(VegaFusionError::internal(&join_err.to_string()))
+                            return Err(VegaFusionError::internal(join_err.to_string()))
                         }
                     };
 
