@@ -1,11 +1,3 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
 use datafusion::scalar::ScalarValue;
 use dssim::{Dssim, DssimImage};
 use regex::Regex;
@@ -54,7 +46,7 @@ impl NodeJsRuntime {
         working_dir.push("vegajs_runtime");
 
         let mut proc = Command::new("node")
-            .args(&["-i", "--experimental-repl-await"])
+            .args(["-i", "--experimental-repl-await"])
             .stdin(Stdio::piped())
             .stdout(Stdio::piped())
             .stderr(Stdio::piped())
@@ -223,7 +215,7 @@ impl VegaJsRuntime {
     {
         let json_str = Self::clean_json_string(json_str);
         match serde_json::from_str(&json_str) {
-            Err(_err) => Err(VegaFusionError::internal(&format!(
+            Err(_err) => Err(VegaFusionError::internal(format!(
                 "Failed to parse result as json:\n{}",
                 json_str
             ))),
@@ -272,7 +264,7 @@ impl VegaJsRuntime {
         // Add datasets (for use in data expressions)
         let mut data = vec![];
         for (name, val) in &config.data_scope {
-            data.push(json!({"name": name, "values": val.to_json()}))
+            data.push(json!({"name": name, "values": val.to_json()?}))
         }
 
         // Create spec
@@ -301,12 +293,12 @@ impl VegaJsRuntime {
         // Add additional dataset from compilation config
         let mut data = Vec::new();
         for (name, val) in &config.data_scope {
-            data.push(json!({"name": name.clone(), "values": val.to_json()}))
+            data.push(json!({"name": name.clone(), "values": val.to_json()?}))
         }
 
         data.push(json!({
             "name": "_dataset",
-            "values": data_table.to_json(),
+            "values": data_table.to_json()?,
             "transform": transforms
         }));
 
@@ -390,7 +382,9 @@ impl VegaJsRuntime {
         let result_str = fs::read_to_string(result_tmppath)
             .with_context(|| format!("Failed to read {}", result_tmppath))?;
 
-        let result_img: ExportImage = serde_json::from_str(&result_str).unwrap();
+        let result_img: ExportImage = serde_json::from_str(&result_str)
+            .with_context(|| format!("Failed to parse as JSON: {}", result_str))?;
+
         Ok(result_img)
     }
 
@@ -499,7 +493,7 @@ impl ExportImage {
         attr.set_save_ssim_maps(1);
         let this_img = self.to_dssim(&attr)?;
         let other_img = other.to_dssim(&attr)?;
-        let (diff, ssim_maps) = attr.compare(&this_img, &other_img);
+        let (diff, ssim_maps) = attr.compare(&this_img, other_img);
         // println!("ssim_map: {:?}", ssim_map);
 
         if diff > 0.0 {
