@@ -1,27 +1,34 @@
-/*
- * VegaFusion
- * Copyright (C) 2022 VegaFusion Technologies LLC
- *
- * This program is distributed under multiple licenses.
- * Please consult the license documentation provided alongside
- * this program the details of the active license.
- */
 use pyo3::exceptions::PyValueError;
 use pyo3::prelude::*;
 use pyo3::types::{PyBytes, PyDict, PyList, PyString};
 use std::collections::HashMap;
+use std::sync::Once;
 use tokio::runtime::Runtime;
 use vegafusion_core::error::{ToExternalError, VegaFusionError};
 use vegafusion_core::proto::gen::pretransform::pre_transform_spec_warning::WarningType;
 use vegafusion_core::proto::gen::pretransform::pre_transform_values_warning::WarningType as ValueWarningType;
-use vegafusion_core::proto::gen::services::pre_transform_spec_result;
 use vegafusion_rt_datafusion::task_graph::runtime::TaskGraphRuntime;
 
+use env_logger::{Builder, Target};
+use pythonize::depythonize;
 use serde::{Deserialize, Serialize};
-use vegafusion_core::data::dataset::VegaFusionDataset;
 use vegafusion_core::proto::gen::tasks::Variable;
+use vegafusion_core::spec::chart::ChartSpec;
 use vegafusion_core::task_graph::graph::ScopedVariable;
 use vegafusion_core::task_graph::task_value::TaskValue;
+use vegafusion_rt_datafusion::data::dataset::VegaFusionDataset;
+use vegafusion_rt_datafusion::tokio_runtime::TOKIO_THREAD_STACK_SIZE;
+
+static INIT: Once = Once::new();
+
+pub fn initialize_logging() {
+    INIT.call_once(|| {
+        // Init logger to write to stdout
+        let mut builder = Builder::from_default_env();
+        builder.target(Target::Stdout);
+        builder.init();
+    });
+}
 
 #[derive(Clone, Serialize, Deserialize)]
 struct PreTransformSpecWarning {
@@ -59,6 +66,8 @@ impl PyTaskGraphRuntime {
         memory_limit: Option<usize>,
         worker_threads: Option<i32>,
     ) -> PyResult<Self> {
+        initialize_logging();
+
         let mut tokio_runtime_builder = tokio::runtime::Builder::new_multi_thread();
         tokio_runtime_builder.enable_all();
 
@@ -66,6 +75,8 @@ impl PyTaskGraphRuntime {
             tokio_runtime_builder.worker_threads(worker_threads.max(1) as usize);
         }
 
+        tokio_runtime_builder.thread_stack_size(TOKIO_THREAD_STACK_SIZE);
+
         // Build tokio runtime
         let tokio_runtime = tokio_runtime_builder
             .build()
@@ -86,72 +97,75 @@ impl PyTaskGraphRuntime {
 
     pub fn pre_transform_spec(
         &self,
-        spec: String,
+        spec: PyObject,
         local_tz: String,
         default_input_tz: Option<String>,
         row_limit: Option<u32>,
+        preserve_interactivity: Option<bool>,
         inline_datasets: &PyDict,
-    ) -> PyResult<(String, String)> {
+    ) -> PyResult<(PyObject, PyObject)> {
         let inline_datasets = deserialize_inline_datasets(inline_datasets)?;
+        let spec = parse_json_spec(spec)?;
+        let preserve_interactivity = preserve_interactivity.unwrap_or(false);
 
-        let response = self
+        let (spec, warnings) = self
             .tokio_runtime
             .block_on(self.runtime.pre_transform_spec(
                 &spec,
                 &local_tz,
                 &default_input_tz,
                 row_limit,
+                preserve_interactivity,
                 inline_datasets,
             ))?;
 
-        match response.result.unwrap() {
-            pre_transform_spec_result::Result::Error(err) => {
-                Err(PyValueError::new_err(format!("{:?}", err)))
-            }
-            pre_transform_spec_result::Result::Response(response) => {
-                let warnings: Vec<_> = response.warnings.iter().map(|warning| {
-                    match warning.warning_type.as_ref().unwrap() {
-                        WarningType::RowLimit(_) => {
-                            PreTransformSpecWarning {
-                                typ: "RowLimitExceeded".to_string(),
-                                message: "Some datasets in resulting Vega specification have been truncated to the provided row limit".to_string()
-                            }
-                        }
-                        WarningType::BrokenInteractivity(_) => {
-                            PreTransformSpecWarning {
-                                typ: "BrokenInteractivity".to_string(),
-                                message: "Some interactive features may have been broken in the resulting Vega specification".to_string()
-                            }
-                        }
-                        WarningType::Unsupported(_) => {
-                            PreTransformSpecWarning {
-                                typ: "Unsupported".to_string(),
-                                message: "Unable to pre-transform any datasets in the Vega specification".to_string()
-                            }
-                        }
-                        WarningType::Planner(warning) => {
-                            PreTransformSpecWarning {
-                                typ: "Planner".to_string(),
-                                message: warning.message.clone()
-                            }
-                        }
+        let warnings: Vec<_> = warnings.iter().map(|warning| {
+            match warning.warning_type.as_ref().unwrap() {
+                WarningType::RowLimit(_) => {
+                    PreTransformSpecWarning {
+                        typ: "RowLimitExceeded".to_string(),
+                        message: "Some datasets in resulting Vega specification have been truncated to the provided row limit".to_string()
                     }
-                }).collect();
-
-                Ok((response.spec, serde_json::to_string(&warnings).unwrap()))
+                }
+                WarningType::BrokenInteractivity(_) => {
+                    PreTransformSpecWarning {
+                        typ: "BrokenInteractivity".to_string(),
+                        message: "Some interactive features may have been broken in the resulting Vega specification".to_string()
+                    }
+                }
+                WarningType::Unsupported(_) => {
+                    PreTransformSpecWarning {
+                        typ: "Unsupported".to_string(),
+                        message: "Unable to pre-transform any datasets in the Vega specification".to_string()
+                    }
+                }
+                WarningType::Planner(warning) => {
+                    PreTransformSpecWarning {
+                        typ: "Planner".to_string(),
+                        message: warning.message.clone()
+                    }
+                }
             }
-        }
+        }).collect();
+
+        Python::with_gil(|py| -> PyResult<(PyObject, PyObject)> {
+            let py_spec = pythonize::pythonize(py, &spec)?;
+            let py_warnings = pythonize::pythonize(py, &warnings)?;
+            Ok((py_spec, py_warnings))
+        })
     }
 
     pub fn pre_transform_datasets(
         &self,
-        spec: String,
+        spec: PyObject,
         variables: Vec<(String, Vec<u32>)>,
         local_tz: String,
         default_input_tz: Option<String>,
+        row_limit: Option<u32>,
         inline_datasets: &PyDict,
-    ) -> PyResult<(PyObject, String)> {
+    ) -> PyResult<(PyObject, PyObject)> {
         let inline_datasets = deserialize_inline_datasets(inline_datasets)?;
+        let spec = parse_json_spec(spec)?;
 
         // Build variables
         let variables: Vec<ScopedVariable> = variables
@@ -170,6 +184,7 @@ impl PyTaskGraphRuntime {
                 &variables,
                 &local_tz,
                 &default_input_tz,
+                row_limit,
                 inline_datasets,
             ))?;
 
@@ -180,11 +195,15 @@ impl PyTaskGraphRuntime {
                     typ: "Planner".to_string(),
                     message: planner_warning.message.clone(),
                 },
+                ValueWarningType::RowLimit(_) => PreTransformSpecWarning {
+                    typ: "RowLimitExceeded".to_string(),
+                    message: "Some datasets in resulting Vega specification have been truncated to the provided row limit".to_string()
+                }
             })
             .collect();
 
-        let response_list: PyObject = Python::with_gil(|py| -> PyResult<PyObject> {
-            let response_list = PyList::empty(py);
+        Python::with_gil(|py| -> PyResult<(PyObject, PyObject)> {
+            let py_response_list = PyList::empty(py);
             for value in values {
                 let bytes: PyObject = if let TaskValue::Table(table) = value {
                     PyBytes::new(py, table.to_ipc_bytes()?.as_slice()).into()
@@ -193,12 +212,12 @@ impl PyTaskGraphRuntime {
                         "Unexpected value type",
                     )));
                 };
-                response_list.append(bytes)?;
+                py_response_list.append(bytes)?;
             }
-            Ok(response_list.into())
-        })?;
 
-        Ok((response_list, serde_json::to_string(&warnings).unwrap()))
+            let py_warnings = pythonize::pythonize(py, &warnings)?;
+            Ok((py_response_list.into(), py_warnings))
+        })
     }
 
     pub fn clear_cache(&self) {
@@ -231,6 +250,31 @@ fn vegafusion_embed(_py: Python, m: &PyModule) -> PyResult<()> {
     Ok(())
 }
 
+/// Helper function to parse an input Python string or dict as a ChartSpec
+fn parse_json_spec(chart_spec: PyObject) -> PyResult<ChartSpec> {
+    Python::with_gil(|py| -> PyResult<ChartSpec> {
+        if let Ok(chart_spec) = chart_spec.extract::<&str>(py) {
+            match serde_json::from_str::<ChartSpec>(chart_spec) {
+                Ok(chart_spec) => Ok(chart_spec),
+                Err(err) => Err(PyValueError::new_err(format!(
+                    "Failed to parse chart_spec string as Vega: {}",
+                    err
+                ))),
+            }
+        } else if let Ok(chart_spec) = chart_spec.cast_as::<PyDict>(py) {
+            match depythonize(chart_spec) {
+                Ok(chart_spec) => Ok(chart_spec),
+                Err(err) => Err(PyValueError::new_err(format!(
+                    "Failed to parse chart_spec dict as Vega: {}",
+                    err
+                ))),
+            }
+        } else {
+            Err(PyValueError::new_err("chart_spec must be a string or dict"))
+        }
+    })
+}
+
 #[cfg(test)]
 mod tests {
     #[test]
